////Demonstrates how to use the Led and Switch non-blocking OOP libraries
//
//#include "led.h"      //Note requires timer.h
//#include "switch.h"
//
//Led led1(2), led2(3), led3(4), led4(5) , led5(6), led6(7);  //Create 6 Led objects on pins 2, 3, 4, 5, 6 and 7
//Switch sw1(8), sw2(9);          //Create two switch objects on pins 8 and 9
//
//void setup () {
//led1.on(1000);    //Turn led1 on and wait for 1000ms (blocking)
//led1.off(1000);   //Turn led1 off and wait for 1000ms (blocking)
//}
//
//void loop () {
//  //Non-blocking means methods return “immediately”
////The main loop runs at full speed and does not wait for any method
//  //All methods appear to execute simultaneously – e.g. led2 and led3 below flash on and off at the same time
//  //Don’t use the delay() function or any blocking methods in your code unless you want all other processes to wait
//  //If you want to change the function of the main loop for any reason you should implement a Finite State Machine. See example.
//  led1.on();      //Turn led1 on (continuously)
//  led2.flash(100,100);    //Flash led2 100ms on 100ms off (non-blocking)
//  led3.flash(100,100);    //Flash led3 100ms on 100ms off (non-blocking)
//  led4.state(sw1.state());    //Turn led4 on (programmatically) with a Boolean variable (here led4 follows the state of sw1)
//  led5.state(sw2.state());    //Turn led5 on (programmatically) with a Boolean variable (here led5 toggles on and off with each press of sw2)
//  if (sw1.count() > 5) {
//    led6.on();    //Turn led6 on if sw1 has been pressed more than 5 times
//  }
//}

//Demonstrates how to use the non-blocking Timer library and a Finite State Machine in the main loop
//Car Push/Pull Automatic Transmission Shift controller
//Up and Down switches select the Park, Reverse and Drive states of the automatic transmission
//Special automatic reversing cutout with warning

#include "timer.h"
#include "led.h"
#include "switch.h"
#include "state.h"

//Define states
#define PARK 1
#define REV 2
#define DRIVE 3

Led park(2), rev(3), drive(4);          //Create 3 Led objects on pins 2, 3 and 4
Switch up(8), down(9);                  //Create 2 Switch objects on pins 8 and 9
Timer t1, t2;                           //Create 2 non-blocking Timer objects
State state(PARK);                      //Create 1 state variable and initialize to PARK

void setup () {}

void loop () {
  //Finite State Machine – The outputs depend on the current state.
  //State transitions depend on the state of timers and external inputs.
  switch (state.current) {
    case PARK:
      if (state.changed()) {
        park.on(); rev.off(); drive.off();          //Turn on the park LED
      }
      if (down.pressed()) state.next(REV);          //Select DRIVE if up is pressed
      break;
    case REV:
      if (state.changed()) {
        t1.reset(10000); t2.reset(7000);            //Reset any timers used in this state
        park.off(); rev.on(); drive.off();          //Set any outputs used in this state
      }
      if (t2.timeout()) rev.flash(100, 100);        //Flash the rev LED if cutout is imminent
      if (t1.timeout()) state.next(PARK);           //Stay in REV for a maximum of 5 seconds
      if (up.pressed()) state.next(PARK);           //Select PARK if up is pressed
      if (down.pressed()) state.next(DRIVE);        //Select DRIVE if down is pressed
      break;
    case DRIVE:
      if (state.changed()) {
        park.off(); rev.off();
      }
      drive.flash(500, 500);                        //Flash the drive LED
      if (up.pressed()) state.next(REV);            //Select REV if up is pressed
      break;
  }
}


